#ifndef AMDIS_VERTEXVECTOR_H
#define AMDIS_VERTEXVECTOR_H

#include "DOFVector.h"

namespace AMDiS {

  class VertexVector : public DOFVectorDOF
  {
  public:
    class Iterator : public DOFIterator<DegreeOfFreedom> {
    public:
      Iterator(VertexVector *c, DOFIteratorType type)
	: DOFIterator<DegreeOfFreedom>(const_cast<DOFAdmin*>(c->getAdmin()), 
				       dynamic_cast<DOFIndexed<DegreeOfFreedom>*>(c), 
				       type)
      {}   
    };

    VertexVector(const DOFAdmin *admin, std::string name);

    ~VertexVector();

    void freeDofIndex(DegreeOfFreedom dof)
    {
      FUNCNAME("VertexVector::freeDofIndex()");
      
      TEST_EXIT_DBG(dof < static_cast<int>(vec.size()))("Should not happen!\n");

      vec[dof] = dof;
    }

    const DOFAdmin *getAdmin() 
    { 
      return admin; 
    }

    void resize(int size) 
    {
      int oldSize = static_cast<int>(vec.size());
      vec.resize(size);
      for (int i = oldSize; i < size; i++)
	vec[i] = i;
    }

    void set(DegreeOfFreedom val);

    void compressDofContainer(int size, std::vector<DegreeOfFreedom> &newDOF) 
    {
      DOFContainer::compressDofContainer(size, newDOF);
      int totalSize = getAdmin()->getSize();
      for (int i = size; i < totalSize; i++)
	vec[i] = i;
    }

    void changeDofIndices(std::map<int, int>& dofIndexMap);

    void print();

  protected:
    const DOFAdmin *admin;
  };

}

#endif
